//////////////////////////////////////////////////////////////////////////////
//
// "Embedded C Programming: Techniques and Applications of C and PIC MCUs"
// by Mark Siegusmund
//
// Exercise 10-1 #1 (part 2)
//
// Creates nodes when the user enters an id number. Each node stores the id
// number and a pointer to the next node. A second list of nodes including
// the issuer of the node is output along side the first list of nodes.
//
//////////////////////////////////////////////////////////////////////////////

#include <e3.h>
#include <stdlibm.h>

#define UNUSED 0xFFFF

typedef node * nodeptr;

typedef struct {
   int32 id_number;
   int32 issuer;
   nodeptr next_issuer;
   nodeptr next;
} node;

node list[20];
int next_node = 0;
nodeptr first_issuer = UNUSED;
nodeptr first = UNUSED;

void sort_list(void);
nodeptr add_node(int id, int issuer);
void print_list(void);

void main(void) 
{
   for(;;) {
      printf("Enter ID Number: ");
      int id = get_int_usb();
      printf("\r\nEnter Issuer: ");
      int issuer = get_int_usb();
      list[next_node++] = add_node(id, issuer);
      sort_list();
      print_list();
   }

   for(;;);
}

nodeptr add_node(int id, int issuer) {
   nodeptr cur;
   
   cur = (nodeptr)malloc(sizeof(node));
   cur -> id_number = id;
   cur -> issuer = issuer;
   cur -> next_issuer = first_issuer;
   cur -> next = first;
   
   first_issuer = cur;
   first = cur;   
   
   return cur;
}

void sort_list(void) {
   nodeptr nxt = first -> next;
   nodeptr ptr = first;
   
   while(nxt != UNUSED)
   {
      while((nxt != ptr) && (nxt->id_number < ptr->id_number)) {
         int tmp = ptr->id_number;
         ptr->id_number = nxt->id_number;
         nxt->id_number = tmp;
         ptr = ptr->next;
      }
      ptr = nxt;
      nxt = nxt->next;
   }
   
   nxt = first_issuer -> next;
   ptr = first_issuer;
   
   while(nxt != UNUSED)
   {
      while((nxt != ptr) && (nxt->issuer < ptr->issuer)) {
         int tmp = ptr->issuer;
         ptr->issuer = nxt->issuer;
         nxt->issuer = tmp;
         ptr = ptr->next;
      }
      ptr = nxt;
      nxt = nxt->next;
   }
}

void print_list(void) {
   nodeptr ptr = first;
   nodeptr issuer_ptr = first_issuer;
   
   while(ptr != UNUSED) {
      printf("\r\n[%d] [%d]\r\n", (int)ptr->id_number, (int)issuer_ptr->issuer);
      ptr = ptr -> next;
      issuer_ptr = issuer_ptr -> next_issuer;
   }
}
